/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

Mercurial = {};
Mercurial = (function($, undef) {

	var mercurial = {
		init : function() {
			var repositoryInfoColumn = $('td.repository-info');

			$.map(repositoryInfoColumn, function(column) {
				mercurial.getRepositoryInfo(column, 0);
			});

			$('a.repository-reload').bind('click', mercurial.refreshRepositoryStatus);
		},

		refreshRepositoryStatus : function(event) {
			event.preventDefault();

			var repositoryID = $(this).attr('rel');
			var column       = $('td.repository-info[rel="' + repositoryID + '"]');

			mercurial.getRepositoryInfo(column, 1);
		},

		getRepositoryInfo : function(column, forceCache) {
			var repositoryID = $(column).attr('rel');
			var requestData  = {
				page         : 'mercurial_ajax',
				func         : 'getRepositoryInfo',
				repositoryID : repositoryID,
				forceCache   : forceCache
			};

			$(column).empty().addClass('loading');

			mercurial.ajax('post', 'json', 'index.php', requestData, function(data) {
				var tags       = data.tags;
				var incoming   = data.incoming;
				var outgoing   = data.outgoing;
				var status     = data.status;
				var summary    = data.summary;

				$('<ul/>').addClass('repository-status').attr('rel', repositoryID).appendTo(column);

				mercurial.createTagInfo(column, repositoryID, tags, summary);
				mercurial.createStatusInfo(column, repositoryID, incoming, 'incoming');
				mercurial.createStatusInfo(column, repositoryID, outgoing, 'outgoing');
				mercurial.createStatusInfo(column, repositoryID, status, 'status');

				$(column).removeClass('loading');
			});
		},

		createTagInfo : function(column, repositoryID, data, summary) {
			var tags       = data.tags;
			var subsequent = data.subsequent;
			var changeset  = data.changeset;
			var link       = $('<a/>').attr({ 'href' : 'index.php?page=mercurial&func=update&id=' + repositoryID, 'title' : summary.user + ' (' + summary.date + '): ' + summary.summary });

			if (tags !== undefined) {
				$.map(tags, function(tag) {
					var container = $('<span/>').addClass('iconized tag');
					var content   = $('<span/>').addClass('iconized-content');

					content.html(tag + ' +' + subsequent);
					content.appendTo(container);
					container.appendTo(link);
					link.appendTo(column);
				});
			}
			else {
				var container = $('<span/>').addClass('iconized changeset');
				var content   = $('<span/>').addClass('iconized-content');

				content.html(changeset + ' +' + subsequent);
				content.appendTo(container);
				container.appendTo(link);
				link.appendTo(column);
			}
		},

		createStatusInfo : function(column, repositoryID, data, type) {
			var wrapper   = $('<li/>');
			var holder    = $('ul.repository-status[rel="' + repositoryID + '"]');
			var link      = $('<a/>').attr({ 'href' : 'index.php?page=mercurial&func=' + type + '&id=' + repositoryID, 'title' : type.ucfirst() });
			var container = $('<span/>').addClass('iconized ' + type);
			var content   = $('<span/>').addClass('iconized-content');

			if (data) {
				wrapper.addClass('affected');
			}

			content.html(data);
			content.appendTo(container);
			container.appendTo(link);
			link.appendTo(wrapper);
			wrapper.appendTo(holder);
		},

		initMercurialTree : function(selector) {
			var pathPrefix    = '/data/dyn/public/mercurial';
			var inputSelector = selector;

			$('div.' + selector).jstree({
				plugins : ['ui', 'themes', 'json_data'],
				core: {
					initially_open : ['#root'],
					animation      : false
				},
				themes: {
					url   : pathPrefix + '/js/jstree/themes/default/style.css',
					dots  : false,
					theme : 'default',
					icons : false
				},
				json_data : {
					ajax : {
						url : 'index.php?page=mercurial_ajax&func=getMercurialTree'
					}
				},
				ui : {
					select_multiple_modifier : 'off',
					select_range_modifier    : 'off'
				}
			}).bind('select_node.jstree', function (event, data) {
				$('input.' + inputSelector + '_hidden').val(data.rslt.obj.attr('url'));
			});
		},

		ajax : function(type, datatype, url, data, callback) {
			$.ajax({
				type     : type,
				dataType : datatype,
				url      : url,
				data     : data,
				success  : callback
			});
		}
	};

	var update = {
		toggleRevisionSelection : function() {
			$('input.filterable').bind('change', function() {
				$('div.node').toggleClass('all');
				$('div.node').removeClass('selected');
				$('div.changesets').scrollTop(0);
			});

			$('div.node').bind('click', function() {
				var node      = $(this);
				var changeset = null;

				$('div.node').not(node).removeClass('selected');

				if (node.hasClass('selected')) {
					node.removeClass('selected');
				}
				else {
					node.addClass('selected');
					changeset = node.attr('rel');
				}

				$('input#changeset').val(changeset);
			});
		}
	};

	mercurial.Update = update;

	$.fn.exists = function() {
		return ($(this).length > 0);
	};

	String.prototype.ucfirst = function() {
		return this.charAt(0).toUpperCase() + this.slice(1);
	};

	var log = function(args) {
		if (typeof(window.console) != 'undefined') {
			window.console.log('Mercurial: ', arguments);
		}
	};

	return mercurial;

})(jQuery, Mercurial);

$(function() {
	Mercurial.init();
});
