<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Clone
 */
class Hg_Command_Clone extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Internal command
	 *
	 * @var string
	 */
	protected $command = 'clone';

	/**
	 * Required options
	 *
	 * @var array
	 */
	protected $requiredOptions = array(
		'noninteractive' => null,
		'files'          => null
	);

	/**
	 * Allowed options
	 *
	 * @var array
	 */
	protected $allowedOptions = array(
	        'noupdate'     => null,
	        'updaterev'    => null,
	        'rev'          => null,
	        'branch'       => null,
	        'pull'         => null,
	        'uncompressed' => null,
	        'ssh'          => null,
	        'remotecmd'    => null,
	        'insecure'     => null
	);

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct($params = null, Hg_Service $service) {
		$this->service = $service;

		if (empty($params[0])) {
			throw new Hg_Exception('The repository you want to clone from must be defined as a string path arugment to clone().');
		}
			else {
				$this->from($params[0]);
			}
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Clone> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status !== 0) {
			throw new Hg_Exception($this->getOutput('external'));
		}

		return $this;
	}

	/**
	 * Set from option
	 *
	 * @param <string> $source Source to set
	 *
	 * @return <Hg_Command_Clone> Instance of self
	 */
	public function from($path) {
		$files    = $this->getOption('files');
		$files[0] = $path;

		$this->addOption('files', $files);

		return $this;
	}

	/**
	 * Set to option
	 *
	 * @param <string> $path Destination path to set
	 *
	 * @return <Hg_Command_Clone> Instance of self
	 */
	public function to($path) {
		$directory = new Hg_Util_Directory($path);

		if ($directory->exists()) {
			throw new Hg_Exception('Cannot clone repository. Directory already exists.');
		}

		$files    = $this->getOption('files');
		$files[1] = $path;

		$this->addOption('files', $files);

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$output = new Hg_Output($this->output);
				$output->format($format);

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}
}
