<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Identify
 */
class Hg_Command_Identify extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Internal command
	 *
	 * @var string
	 */
	protected $command = 'identify';

	/**
	 * Required options
	 *
	 * @var array
	 */
	protected $requiredOptions = array(
		'noninteractive' => null,
		'repository'     => null
	);

	/**
	 * Allowed options
	 *
	 * @var array
	 */
	protected $allowedOptions = array(
		'files' => null,
		'tags'  => null,
		'rev'   => null,
		'id'    => null
	);

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct($params = null, Hg_Service $service) {
		$this->service = $service;

		if (empty($params[0]) === false) {
			$this->from($params[0]);
		}
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Identify> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null,
			'repository'     => $this->service->repository->getPath()
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status > 1) {
			throw new Hg_Exception($this->getOutput('external'));
		}

		return $this;
	}

	/**
	 * Set tags option
	 *
	 * @param <string> $tags Option tags
	 *
	 * @return <Hg_Command_Identify> Instance of self
	 */
	public function tags($tags) {
		if ($tags) {
			$this->addOption('tags', null);
		}

		return $this;
	}

	/**
	 * Set from option
	 *
	 * @param <string> $source Source to set
	 *
	 * @return <Hg_Command_Identify> Instance of self
	 */
	public function from($source) {
		if (empty($source)) {
			throw new Hg_Exception('The source to show identify from cannot be empty.');
		}

		$this->addOption('files', $source);

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$external = new Hg_Output($this->output);
				$external->format($format);

				$output = $external;

				break;

			case 'internal':
				$output = $this->output[0];

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}
}
