<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Status
 */
class Hg_Command_Status extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Internal command
	 *
	 * @var string
	 */
	protected $command = 'status';

	/**
	 * Required options
	 *
	 * @var array
	 */
	protected $requiredOptions = array(
		'noninteractive' => null,
		'repository'     => null
	);

	/**
	 * Allowed options
	 *
	 * @var array
	 */
	protected $allowedOptions = array(
		'all'      => null,
		'modified' => null,
		'added'    => null,
		'removed'  => null,
		'deleted'  => null,
		'clean'    => null,
		'unknown'  => null,
		'ignored'  => null,
		'rev'      => null
	);

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct(array $params = array(), Hg_Service $service) {
		$this->service = $service;

		if (empty($params) === false) {
			$this->setOptions($params);
		}
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Status> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null,
			'repository'     => $this->service->repository->getPath()
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status > 1) {
			throw new Hg_Exception($this->getOutput('external'));
		}

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$output = new Hg_Output($this->output);
				$output->format($format);

				break;

			case 'internal':
				$output = $this->output;

				$output = preg_replace('/^([\?]) (.*)/i', '<p><span class="unknown">$1</span>$2</p>',  $output);
				$output = preg_replace('/^([C]) (.*)/i',  '<p><span class="clean">$1</span>$2</p>',    $output);
				$output = preg_replace('/^([R]) (.*)/i',  '<p><span class="removed">$1</span>$2</p>',  $output);
				$output = preg_replace('/^([A]) (.*)/i',  '<p><span class="added">$1</span>$2</p>',    $output);
				$output = preg_replace('/^([M]) (.*)/i',  '<p><span class="modified">$1</span>$2</p>', $output);
				$output = preg_replace('/^([\!]) (.*)/i', '<p><span class="missing">$1</span>$2</p>',  $output);
				$output = preg_replace('/^([I]) (.*)/i',  '<p><span class="ignored">$1</span>$2</p>',  $output);
				$output = preg_replace('/^\s+/', '', $output);

				$output = implode($output);

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}
}
