<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Summary
 */
class Hg_Command_Summary extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Internal command
	 *
	 * @var string
	 */
	protected $command = 'summary';

	/**
	 * Required options
	 *
	 * @var array
	 */
	protected $requiredOptions = array(
		'noninteractive' => null,
		'repository'     => null
	);

	/**
	 * Allowed options
	 *
	 * @var array
	 */
	protected $allowedOptions = array(
		'remote' => null
	);

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct(array $params = array(), Hg_Service $service) {
		$this->service = $service;

		if (empty($params) === false) {
			$this->setOptions($params);
		}
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Summary> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null,
			'repository'     => $this->service->repository->getPath()
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status > 1) {
			throw new Hg_Exception($this->getOutput('external'));
		}

		return $this;
	}

	/**
	 * Set remote option
	 *
	 * @return <Hg_Command_Summary> Instance of self
	 */
	public function remote($remote = false) {
		if ($remote) {
			$this->addOption('remote', null);
		}

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$external = new Hg_Output($this->output);
				$external->format($format);

				$output = $external;

				break;

			case 'internal':
				$internal = array();

				foreach ($this->output as $line) {
					if (preg_match('#^parent:\s(\d+):([0-9a-f]+)\s*(.*?)$#i', $line, $match)) {
						$internal['rev']       = $match[1];
						$internal['changeset'] = $match[2];
						$internal['tags']      = $match[3];
					}

					if (preg_match('#^\s(.+?)$#i', $line, $match)) {
						$internal['summary'] = $match[1];
					}

					if (preg_match('#^branch:\s(.+?)$#i', $line, $match)) {
						$internal['banch'] = $match[1];
					}

					if (preg_match('#^commit:\s(.+?)$#i', $line, $match)) {
						$internal['commit'] = $match[1];
					}

					if (preg_match('#^update:\s(.+?)$#i', $line, $match)) {
						$internal['update'] = $match[1];
					}

					if (preg_match('#^remote:\s(.+?)$#i', $line, $match)) {
						$internal['remote'] = $match[1];
					}
				}

				$output = $internal;

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}
}
