<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command_Tags
 */
class Hg_Command_Tags extends Hg_Command implements Hg_Interface_Command {
	/**
	 * Internal command
	 *
	 * @var string
	 */
	protected $command = 'tags';

	/**
	 * Required options
	 *
	 * @var array
	 */
	protected $requiredOptions = array(
		'noninteractive' => null,
		'repository'     => null
	);

	/**
	 * Allowed options
	 *
	 * @var array
	 */
	protected $allowedOptions = array();

	/**
	 * Extended tags
	 *
	 * @var boolean
	 */
	protected $extended = false;

	/**
	 * Constructor
	 *
	 * @param <mixed> $params Mixed parameters
	 * @param <Hg_Service> $service Instance of Hg_Service
	 */
	public function __construct(array $params = array(), Hg_Service $service) {
		$this->service = $service;

		if (empty($params) === false) {
			$this->setOptions($params);
		}
	}

	/**
	 * Execute command
	 *
	 * @param <array> $params Array of parameters
	 *
	 * @return <Hg_Command_Tags> Instance of self
	 */
	public function execute(array $params = array()) {
		if (empty($params) === false) {
			$this->setOptions($params);
		}

		$this->addOptions(array(
			'noninteractive' => null,
			'repository'     => $this->service->repository->getPath()
		));

		$this->setCommandString();

		exec($this->commandString, $this->output, $this->status);

		if ($this->status > 1) {
			throw new Hg_Exception($this->status);
		}

		return $this;
	}

	/**
	 * Extend tags with more detailed informations
	 *
	 * @return <Hg_Command_Tags> Instance of self
	 */
	public function extended($extended) {
		$this->extended = $extended;

		return $this;
	}

	/**
	 * Get status
	 *
	 * @return <int> Status return from executed command
	 */
	public function getStatus() {
		return $this->status;
	}

	/**
	 * Get command string
	 *
	 * @return <string> Command string which was executed
	 */
	public function getCommand() {
		return $this->commandString;
	}

	/**
	 * Get output
	 *
	 * @param <string> $type Type of the output
	 * @param <string> $format Format of the output
	 *
	 * @return <mixed> Formatted output by type and format
	 */
	public function getOutput($type = null, $format = null) {
		static $output = null;

		switch ($type) {
			case 'external':
				$output = new Hg_Output($this->output);
				$output->format($format);

				break;
			case 'internal':
				$internal = array();

				foreach (array_filter($this->output) as $line) {
					if (preg_match('#^(.+?)\s+(\d+):([0-9a-f]+)$#i', $line, $match)) {
						if ($this->extended) {
							$log     = $this->service->log(array('rev' => $match[3]))->run()->getOutput();
							$summary = '';
							$date    = 0;

							foreach ($log as $revision) {
								if (preg_match('#^summary:\s*(.*?)$#i', $revision, $lmatch)) {
									$summary = $lmatch[1];
								}
								elseif (preg_match('#^date:\s*(.*?)$#i', $revision, $lmatch)) {
									$date = strtotime($lmatch[1]);
								}
							}

							$internal[$match[1]] = array(
								'changeset' => $match[3],
								'summary'   => $summary,
								'date'      => $date
							);
						}
						else {
							$internal[$match[3]][] = $match[1];
						}
					}
				}

				$output = $internal;

				break;

			default:
				$output = $this->output;

				break;
		}

		return $output;
	}
}
