<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial_Provider class
 */
class Mercurial_Provider {
	/**
	 * Get repositories
	 *
	 * @param <string> $where Database where condition
	 * @param <string> $orderBy Database orderBy condition
	 * @param <string> $direction Database direction condition
	 * @param <int> $offset Database offset condition
	 * @param <int> $limit Database limit condition
	 *
	 * @return <array> ID array of all existing repositories in database
	 */
	public static function getRepositories($where = '1', $orderBy = 'title', $direction = 'asc', $offset = 0, $limit = -1) {
		$database  = sly_DB_Persistence::getInstance();
		$prefix    = sly_Core::config()->get('DATABASE/TABLE_PREFIX');
		$direction = strtolower($direction) == 'desc' ? 'DESC' : 'ASC';
		$offset    = abs((int) $offset);
		$limit     = $limit < 0 ? '18446744073709551615' : (int) $limit;
		$query     = sprintf('SELECT repository_id FROM %smercurial_repository WHERE %s ORDER BY %s %s LIMIT %s, %s', $prefix, $where, $orderBy, $direction, $offset, $limit);

		$database->query($query);

		$result = array();

		foreach ($database as $row) {
			$result[$row['repository_id']] = Mercurial_Provider::getRepository($row['repository_id']);
		}

		return $result;
	}

	/**
	 * Get total repositories
	 *
	 * @param $where <string> Database where clause
	 *
	 * @return <int> Total count of repositories in database
	 */
	public static function getTotalRepositories($where = '1') {
		$database = sly_DB_Persistence::getInstance();
		$prefix   = sly_Core::config()->get('DATABASE/TABLE_PREFIX');
		$query    = sprintf('SELECT repository_id FROM %smercurial_repository WHERE %s', $prefix, $where);

		$database->query($query);

		$total = $database->affectedRows();
		$total = $total === false ? -1 : (int) $total;

		return $total;
	}

	/**
	 * Get repository
	 *
	 * @param $repositoryID <int> ID of an existing repository
	 *
	 * @return <Mercurial_Model_Repository> Instance of a mercurial repository model
	 */
	public static function getRepository($repositoryID) {
		return Mercurial_Model_Repository::getInstance($repositoryID);
	}
}
