<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial service repository class
 */
class Mercurial_Service_Repository extends Mercurial_Service {
	/**
	 * Repository
	 *
	 * @var <Mercurial_Model_Repository>
	 */
	protected $repository = null;

	/**
	 * Constructor
	 *
	 * @param <Mercurial_Model_Repository> $repository Repository
	 */
	public function __construct(Mercurial_Model_Repository $repository = null) {
		$this->repository = $repository;
	}

	/**
	 * Service Method to create Repository Database Entry
	 *
	 * @param <array> $data Array of repository container data
	 *
	 * @return <Mercurial_Model_Repository> Instance of a repository model
	 */
	public function create($data) {
		$database   = sly_DB_Persistence::getInstance();
		$dispatcher = sly_Core::dispatcher();

		$database->insert('mercurial_repository', $data);

		$this->repository = Mercurial_Model_Repository::getInstance($database->lastId());

		$dispatcher->notify('MERCURIAL_REPOSITORY_RECORD_CREATE', $this->repository->getRepositoryID(), array(
			'repository_id ' => $this->repository->getRepositoryID(),
			'title'          => $this->repository->getTitle(),
			'directory'      => $this->repository->getDirectory()
		));

		return $this->repository;
	}

	/**
	 * Service Method to edit Repository Database Entry
	 *
	 * @param <Mercurial_Model_Repository> $repository Instance of a repository model
	 * @param <array> $data Array of repository container data
	 *
	 * @return <Mercurial_Model_Repository> Instance of a repository model
	 */
	public function update($data) {
		$database   = sly_DB_Persistence::getInstance();
		$dispatcher = sly_Core::dispatcher();

		$database->update('mercurial_repository', $data, array('repository_id' => $this->repository->getRepositoryID()));

		$dispatcher->notify('MERCURIAL_REPOSITORY_RECORD_CREATE', $this->repository->getRepositoryID(), array(
			'repository_id ' => $this->repository->getRepositoryID(),
			'title'          => $this->repository->getTitle(),
			'directory'      => $this->repository->getDirectory()
		));

		return $this->repository;
	}

	/**
	 * Service Method to Delete Repository Database Entry
	 *
	 * @param <boolean> $deleteFolder Delete repository folder
	 */
	public function delete($deleteFolder = false) {
		if ($deleteFolder) {
			$this->repository->getHgService()->deleteFolder();
		}

		$database   = sly_DB_Persistence::getInstance();
		$dispatcher = sly_Core::dispatcher();

		$database->delete('mercurial_repository', array('repository_id' => $this->repository->getRepositoryID()));

		$dispatcher->notify('MERCURIAL_REPOSITORY_RECORD_DELETE', $this->repository->getRepositoryID(), array(
			'repository_id ' => $this->repository->getRepositoryID(),
			'title'          => $this->repository->getTitle(),
			'directory'      => $this->repository->getDirectory()
		));
	}
}
